@@ -12,6 +12,10 @@
  
 	0.21	1999-10-28	schoen
 		corrected mem dealloc bug in r3_del_func
+
+	0.30	1999-11-05	schoen
+		don't allocate interface buffer for tables and exceptions
+		added support for R/3 pre 40A
  
 */
 
@@ -40,9 +44,24 @@ typedef struct {
 	RFC_CHAR Paramtext[79]; 
 } RFC_FUNINT;
 
-static RFC_TYPEHANDLE handleOfRFC_FUNINT;
+typedef struct {
+	RFC_CHAR Paramclass[1];
+	RFC_CHAR Parameter[30];
+	RFC_CHAR Tabname[10];
+	RFC_CHAR Fieldname[10];
+	RFC_CHAR Exid[1];
+	RFC_INT Position;
+	RFC_INT Offset;
+	RFC_INT Intlength;
+	RFC_INT Decimals;
+	RFC_CHAR Default[21];
+	RFC_CHAR Paramtext[79]; 
+} RFC3FUNINT;
 
-static RFC_TYPE_ELEMENT typeOfRFC_FUNINT[] = {
+static RFC_TYPEHANDLE h_RFC_FUNINT;
+static RFC_TYPEHANDLE h_RFC3FUNINT;
+
+static RFC_TYPE_ELEMENT t_RFC_FUNINT[] = {
   {"PARAMCLASS", TYPC, 1, 0},
   {"PARAMETER", TYPC, 30, 0},
   {"TABNAME", TYPC, 30, 0},
@@ -55,9 +74,24 @@ static RFC_TYPE_ELEMENT typeOfRFC_FUNINT[] = {
   {"DEFAULT", TYPC, 21, 0},
   {"PARAMTEXT", TYPC, 79, 0}
 };
+
+static RFC_TYPE_ELEMENT t_RFC3FUNINT[] = {
+  {"PARAMCLASS", TYPC, 1, 0},
+  {"PARAMETER", TYPC, 30, 0},
+  {"TABNAME", TYPC, 10, 0},
+  {"FIELDNAME", TYPC, 10, 0},
+  {"EXID", TYPC, 1, 0},
+  {"POSITION", TYPINT, sizeof(RFC_INT), 0},
+  {"OFFSET", TYPINT, sizeof(RFC_INT), 0},
+  {"INTLENGTH", TYPINT, sizeof(RFC_INT), 0},
+  {"DECIMALS", TYPINT, sizeof(RFC_INT), 0},
+  {"DEFAULT", TYPC, 21, 0},
+  {"PARAMTEXT", TYPC, 79, 0}
+};
+
 #define ENTRIES(tab) (sizeof(tab)/sizeof((tab)[0]))
 
-H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
+static H_R3RFC_FUNC r3_pre4_new_func(H_R3RFC_CONN h_conn,
 			char * function_name)
 {
 	static H_R3RFC_FUNC h;
@@ -70,16 +104,16 @@ H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
 	char *RfcException = NULL;
 	ITAB_H thParams;
 	int i;
-	RFC_FUNINT * tParams;
+	RFC3FUNINT * tParams;
 
 	/* install structures */
-	if (handleOfRFC_FUNINT==0) 
+	if (h_RFC3FUNINT==0) 
 	{
 		RfcRc = RfcInstallStructure(
-				"RFC_FUNINT",
-				typeOfRFC_FUNINT,
-                                ENTRIES(typeOfRFC_FUNINT),
-                                &handleOfRFC_FUNINT);
+				"RFC3FUNINT",
+				t_RFC3FUNINT,
+                                ENTRIES(t_RFC3FUNINT),
+                                &h_RFC3FUNINT);
 		if (RfcRc != RFC_OK) 
 		{
 			if (RfcRc == RFC_MEMORY_INSUFFICIENT)
@@ -113,7 +147,7 @@ H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
 	Importing[0].name = NULL;
 
 	/* define internal tables */
-	thParams = ItCreate("PARAMS",sizeof(RFC_FUNINT),0,0);
+	thParams = ItCreate("PARAMS",sizeof(RFC3FUNINT),0,0);
 	if (thParams==ITAB_NULL) 
 	{
 		r3_set_rfcapi_exception("RFC_MEMORY_INSUFFICIENT");
@@ -122,8 +156,9 @@ H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
 
 	Tables[0].name     = "PARAMS";
 	Tables[0].nlen     = 6;
-	Tables[0].type     = handleOfRFC_FUNINT;
+	Tables[0].type     = h_RFC3FUNINT;
 	Tables[0].ithandle = thParams;
+	Tables[0].leng = sizeof(RFC3FUNINT);
 
 	Tables[1].name = NULL;
 
@@ -244,14 +279,22 @@ H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
 			sizeof(tParams->Paramtext));
 		r3_stbl(h->interface[i].paramtext);
 
-		h->interface[i].buffer = malloc(h->interface[i].intlength);
-		if (!h->interface[i].buffer)
-		{
-			r3_set_rfcapi_exception("MALLOC_FAILED");
-			r3_del_func(h);
-			ItDelete(thParams);
-			return NULL;
+
+		if (h->interface[i].paramclass[0] == 'I' ||
+			h->interface[i].paramclass[0] == 'E')
+		{	
+			h->interface[i].buffer = 
+				malloc(h->interface[i].intlength);
+			if (!h->interface[i].buffer)
+			{
+				r3_set_rfcapi_exception("MALLOC_FAILED");
+				r3_del_func(h);
+				ItDelete(thParams);
+				return NULL;
+			}
 		}
+		else
+			h->interface[i].buffer = NULL;
 	}
 
 	/* delete table */
@@ -269,6 +312,237 @@ H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
 	return h;
 }
 
+static H_R3RFC_FUNC r3_4_new_func(H_R3RFC_CONN h_conn,
+			char * function_name)
+{
+	static H_R3RFC_FUNC h;
+	static RFC_FUNCTIONNAME eFuncname;
+	static SYST_LANGU eLanguage;
+	RFC_PARAMETER Exporting[3];
+	RFC_PARAMETER Importing[1];
+	RFC_TABLE Tables[2];
+	RFC_RC RfcRc;
+	char *RfcException = NULL;
+	ITAB_H thParams;
+	int i;
+	RFC_FUNINT * tParams;
+
+	/* install structures */
+	if (h_RFC_FUNINT==0) 
+	{
+		RfcRc = RfcInstallStructure(
+				"RFC_FUNINT",
+				t_RFC_FUNINT,
+                                ENTRIES(t_RFC_FUNINT),
+                                &h_RFC_FUNINT);
+		if (RfcRc != RFC_OK) 
+		{
+			if (RfcRc == RFC_MEMORY_INSUFFICIENT)
+				r3_set_rfcapi_exception("RFC_MEMORY_INSUFFICIENT");
+			else
+				r3_set_rfcapi_exception("UNKNOWN_ERROR");
+			return NULL;	
+		}
+	}
+	/* define export params */
+	memset(eFuncname,' ',sizeof(eFuncname));
+	strncpy(eFuncname, function_name, strlen(function_name));
+	memset(eLanguage,' ',sizeof(eLanguage));
+	eLanguage[0]='E'; /* this shouldn't be hardcoded */
+
+	Exporting[0].name = "FUNCNAME";
+	Exporting[0].nlen = 8;
+	Exporting[0].type = TYPC;
+	Exporting[0].leng = sizeof(RFC_FUNCTIONNAME);
+	Exporting[0].addr = eFuncname;
+
+	Exporting[1].name = "LANGUAGE";
+	Exporting[1].nlen = 8;
+	Exporting[1].type = TYPC;
+	Exporting[1].leng = sizeof(SYST_LANGU);
+	Exporting[1].addr = eLanguage;
+
+	Exporting[2].name = NULL;
+
+	/* no import parameters */
+	Importing[0].name = NULL;
+
+	/* define internal tables */
+	thParams = ItCreate("PARAMS",sizeof(RFC_FUNINT),0,0);
+	if (thParams==ITAB_NULL) 
+	{
+		r3_set_rfcapi_exception("RFC_MEMORY_INSUFFICIENT");
+		return NULL;
+	}
+
+	Tables[0].name     = "PARAMS";
+	Tables[0].nlen     = 6;
+	Tables[0].type     = h_RFC_FUNINT;
+	Tables[0].ithandle = thParams;
+	Tables[0].leng     = sizeof(RFC_FUNINT);
+
+	Tables[1].name = NULL;
+
+	/* call function module */
+	RfcRc = RfcCallReceive(h_conn->h_rfc,
+			"RFC_GET_FUNCTION_INTERFACE",
+			Exporting,
+			Importing,
+			Tables,
+			&RfcException);
+	if (RfcRc != RFC_OK)
+	{
+		switch (RfcRc)
+		{
+		case RFC_FAILURE:
+			r3_set_rfc_sys_exception("RFC_FAILURE");
+			break;
+		case RFC_EXCEPTION:
+			r3_set_rfc_exception(RfcException);
+			break;
+		case RFC_SYS_EXCEPTION:
+			r3_set_rfc_sys_exception(RfcException);
+			break;
+		case RFC_CALL:
+			r3_set_rfc_exception("RFC_CALL");
+			break;
+		default:
+			r3_set_rfcapi_exception("UNKNOWN_ERROR");
+		}
+		ItDelete(thParams);
+		return NULL;
+	}
+
+	/* allocate memory for the function */
+	if (!(h=malloc(sizeof(R3RFC_FUNC))))
+	{
+		r3_set_rfcapi_exception("MALLOC_FAILED");
+		ItDelete(thParams);
+		return NULL;
+	}
+	memset(h, 0, sizeof(R3RFC_FUNC));
+	h->h_conn=h_conn;
+
+        /* get interface */
+	strcpy(h->name, function_name);
+	h->n_interface = ItFill(thParams);	
+	h->interface = malloc(h->n_interface*sizeof(R3RFC_FUNCINT));
+	if (h->interface==NULL)
+	{
+		r3_set_rfcapi_exception("MALLOC_FAILED");
+		r3_del_func(h);
+		ItDelete(thParams);
+		return NULL;
+	}
+	memset(h->interface, 0, h->n_interface*sizeof(R3RFC_FUNCINT));
+        for (i=0; i<h->n_interface; i++) 
+	{
+		/* ABAP arrays are 1-based */
+		tParams = ItGetLine(thParams, i+1); 
+		if (tParams == NULL)
+		{
+			r3_set_rfcapi_exception("RFC_MEMORY_INSUFFICIENT");
+			r3_del_func(h);
+			ItDelete(thParams);
+			return NULL;
+		}
+
+		strncpy(h->interface[i].paramclass,
+			tParams->Paramclass,
+			sizeof(tParams->Paramclass));
+		r3_stbl(h->interface[i].paramclass);
+
+		switch (h->interface[i].paramclass[0])
+		{
+			case 'I':
+				h->n_exporting++;
+				break;
+			case 'E':
+				h->n_importing++;
+				break;
+			case 'T':
+				h->n_tables++;
+				break;
+		}
+
+		strncpy(h->interface[i].parameter,
+			tParams->Parameter,
+			sizeof(tParams->Parameter));
+		r3_stbl(h->interface[i].parameter);
+
+		strncpy(h->interface[i].tabname,
+			tParams->Tabname,
+			sizeof(tParams->Tabname));
+		r3_stbl(h->interface[i].tabname);
+
+		strncpy(h->interface[i].fieldname,
+			tParams->Fieldname,
+			sizeof(tParams->Fieldname));
+		r3_stbl(h->interface[i].fieldname);
+
+		strncpy(h->interface[i].exid,
+			tParams->Exid,
+			sizeof(tParams->Exid));
+		r3_stbl(h->interface[i].exid);
+			
+		h->interface[i].position=tParams->Position;
+		h->interface[i].offset=tParams->Offset;
+		h->interface[i].intlength=tParams->Intlength;
+		h->interface[i].decimal=tParams->Decimals;
+
+		strncpy(h->interface[i].zdefault,
+			tParams->Default,
+			sizeof(tParams->Default));
+		r3_stbl(h->interface[i].zdefault);
+
+		strncpy(h->interface[i].paramtext,
+			tParams->Paramtext,
+			sizeof(tParams->Paramtext));
+		r3_stbl(h->interface[i].paramtext);
+
+
+		if (h->interface[i].paramclass[0] == 'I' ||
+			h->interface[i].paramclass[0] == 'E')
+		{	
+			h->interface[i].buffer = 
+				malloc(h->interface[i].intlength);
+			if (!h->interface[i].buffer)
+			{
+				r3_set_rfcapi_exception("MALLOC_FAILED");
+				r3_del_func(h);
+				ItDelete(thParams);
+				return NULL;
+			}
+		}
+		else
+			h->interface[i].buffer = NULL;
+	}
+
+	/* delete table */
+	ItDelete(thParams);
+
+	h->exporting=malloc((1+h->n_exporting)*sizeof(RFC_PARAMETER));
+	h->importing=malloc((1+h->n_importing)*sizeof(RFC_PARAMETER));
+	h->tables=malloc((1+h->n_tables)*sizeof(RFC_TABLE));
+	if (!h->exporting || !h->importing || !h->tables)
+	{
+		r3_set_rfcapi_exception("MALLOC_FAILED");
+		r3_del_func(h);
+		return NULL;
+	}
+	return h;
+}
+
+H_R3RFC_FUNC r3_new_func(H_R3RFC_CONN h_conn,
+			char * function_name)
+{
+	/* different table and field name length in R/3 4.X and 3.X */
+	if (h_conn->pre4)
+		return r3_pre4_new_func(h_conn, function_name);
+	else
+		return r3_4_new_func(h_conn, function_name);
+}
+
 void r3_del_func(H_R3RFC_FUNC h)
 {
 	int i;
